WORD_TOKENS = '!"#$%&\'()*+,-./:;<=>?@[\\]^_`{|}~\n '


class BaseTokenizer(object):
    def tokenize(self, text):
        pass


class SpaceTokenizer(BaseTokenizer):
    """
    A simple tokenizer where word are separated by spaces.

    Examples
    --------
    >>> SpaceTokenizer().tokenize("This is a-text !")
    ['This', 'is', 'a-text', '!']
    """

    def tokenize(self, text):
        return text.split(" ")


class CharTokenizer(BaseTokenizer):
    """
    A tokenizer where word are separated by special characters.

    Examples
    --------
    >>> CharTokenizer().tokenize("This is a-text !")
    ['This', 'is', 'a', 'text']
    """

    def __init__(self, delimiters=WORD_TOKENS):
        self.delimiters = delimiters

    def tokenize(self, text):
        tokenized_text = []
        part_of_word = ""

        for character in text:
            if character in self.delimiters:
                if part_of_word != "":
                    tokenized_text.append(part_of_word)
                    part_of_word = ""
            else:
                part_of_word += character

        if part_of_word != "":
            tokenized_text.append(part_of_word)

        return tokenized_text


class SafeCharTokenizer(CharTokenizer):
    """
    Same as CharTokenizer, but save save punctuation. Used for generation tasks.

    Examples
    --------
    >>> SafeCharTokenizer().tokenize("This is a-text !")
    ['This ', 'is ', 'a-', 'text !']
    """

    def tokenize(self, text):
        split_regular_char = False
        tokenized_text = []
        part_of_word = ""

        for character in text:
            if split_regular_char:
                if part_of_word != "" and character not in self.delimiters:
                    tokenized_text.append(part_of_word)
                    part_of_word = ""
                    split_regular_char = False

            elif character in self.delimiters:
                split_regular_char = True

            part_of_word += character

        if part_of_word != "":
            tokenized_text.append(part_of_word)

        return tokenized_text


from collections import OrderedDict
from sklearn.svm import SVC
from .tokenizers import CharTokenizer
from .vectorizers import DocVectorizer


class BaseClassifier(object):

    def __init__(self, tokenizer=CharTokenizer(), vectorizer=DocVectorizer()):

        self.vectorizer = vectorizer
        self.vectorizer.tokenizer = tokenizer

    def fit(self, documents):
        """
        :param documents: a dict containing text as keys and label as values
        """
        documents = OrderedDict(documents)

        self.labels = list(documents.values())
        self.text_vectors = self.vectorizer.fit_transform(list(documents.keys()))

    def predict(self, documents):
        """
        :param documents: a list of document to classify.
        """
        pass


class SVMClassifier(BaseClassifier):

    def fit(self, documents):
        BaseClassifier.fit(self, documents)

        self.sk_model = SVC().fit(self.text_vectors.values(), self.labels)

    def predict(self, documents):
        unique_word_save = self.vectorizer.unique_words
        self.vectorizer.fit(documents)
        self.vectorizer.unique_words = unique_word_save

        predict_vectors = self.vectorizer.transform()

        return dict(zip(documents,self.sk_model.predict(predict_vectors.values())))



from .tokenizers import SafeCharTokenizer
from random import choice


class BaseGenerator(object):
    """
    A base class for building text generators.
    """

    def __init__(self, start_end_tokens=('START', 'END'), tokenizer=SafeCharTokenizer()):
        """
        :param start_end_tokens: A tuple that contain start and end tokens.
        """

        self.tokenizer = tokenizer
        self.start_token = start_end_tokens[0]
        self.end_token = start_end_tokens[-1]

        self.documents = []

    def fit(self, documents):
        """Prepare and save the documents."""

        if isinstance(documents, list):
            for doc in documents:
                self.documents.append([self.start_token] + self.tokenizer.tokenize(doc) + [self.end_token])
        else:
            self.documents.append([self.start_token] + self.tokenizer.tokenize(documents) + [self.end_token])

    def predict(self, max_word=30):
        pass


class MarkovGenerator(BaseGenerator):

    def fit(self, documents):
        BaseGenerator.fit(self, documents)
        self.markov_chain = dict()

        # Generate a basic markov chain
        for corpus in self.documents:
            for index, word in enumerate(corpus):
                if not word == self.end_token:
                    if word in self.markov_chain.keys():
                        self.markov_chain[word].append(corpus[index+1])
                    else:
                        self.markov_chain[word] = [corpus[index + 1]]
                else:
                    self.markov_chain[word] = []

    def predict(self, max_word=30, join_chain=""):
        next_word = self.start_token
        generated_suite = []

        for i in range(max_word):
            next_word = choice(self.markov_chain[next_word])
            if next_word == self.end_token:
                break
            else:
                generated_suite.append(next_word)

        return join_chain.join(generated_suite)



from distutils.core import setup


try:
    readme = open('README', 'r').read()
except:
    readme = "Kadot, unsupervised natural language processing."


setup(
    name='Kadot',
    version='0.1.6b',
    packages=['kadot'],
    url='https://github.com/the-new-sky/Kadot',
    long_description=readme,
    download_url='https://github.com/the-new-sky/Kadot/archive/0.1.2.tar.gz',
    install_requires=['numpy', 'scipy', 'sklearn'],
    license='MIT',
    author='the_new_sky',
    author_email='lorisazerty gmail.com',
    description='Kadot, unsupervised natural language processing.',
    keywords=['natural language processing', 'text classification', 'text generation', 'tokenizer', 'word embeddings'],
    classifiers=['Development Status :: 3 - Alpha', 'Intended Audience :: Developers',
                 'License :: OSI Approved :: MIT License', 'Programming Language :: Python :: 3.5',
                 'Programming Language :: Python :: 3.6', 'Topic :: Scientific/Engineering :: Artificial Intelligence',
                 'Topic :: Text Processing :: General']
)
